"""
Django settings for geder project.

Generated by 'django-admin startproject' using Django 5.0.3.

For more information on this file, see
https://docs.djangoproject.com/en/5.0/topics/settings/

For the full list of settings and their values, see
https://docs.djangoproject.com/en/5.0/ref/settings/
"""

import os
from datetime import timedelta
from pathlib import Path

from __settings_local import (
    APPLE_CLIENT_ID,
    APPLE_KEY_ID,
    APPLE_PRIVATE_KEY_PATH,
    APPLE_TEAM_ID,
    BASE_URL,
    LOG_FILE_PATH,
    LOGGING_LEVEL,
    MY_STRAVA_CLIENT_ID,
    MY_STRAVA_CLIENT_SECRET,
)

# Build paths inside the project like this: BASE_DIR / 'subdir'.
BASE_DIR = Path(__file__).resolve().parent.parent
TEMPLATE_DIR = os.path.join(BASE_DIR, "templates")

STATICFILES_DIRS = [
    os.path.join(BASE_DIR, "assets"),
    os.path.join(BASE_DIR, "node_modules"),
]
STATIC_ROOT = BASE_DIR / "static"
STATIC_URL = "static/"

# Quick-start development settings - unsuitable for production
# See https://docs.djangoproject.com/en/5.0/howto/deployment/checklist/


def get_secret(key: str):
    if os.getenv(key.upper()):
        return os.getenv(key.upper())
    elif os.path.exists(f"/etc/{key.lower()}"):
        with open(f"/etc/{key.lower()}") as f:
            return f.read().strip()
    return None


# SECURITY WARNING: keep the secret key used in production secret!
# SECRET_KEY = 'django-insecure-q6crt0tc9*m(kb2ygurwd1x&td@oshhtqwh3%i^wgm@c^w!2&s'
SECRET_KEY = get_secret("SECRET_DJANGO_GEDER_AT_KEY")  # secret_django_geder_at_key
# SECURITY WARNING: don't run with debug turned on in production!
DEBUG = True

ALLOWED_HOSTS = ["*"]


# Application definition

INSTALLED_APPS = [
    "django.contrib.admin",
    "django.contrib.auth",
    "django.contrib.contenttypes",
    "django.contrib.sessions",
    "django.contrib.messages",
    "django.contrib.staticfiles",
    #
    # 3rd party
    "bootstrap4",
    #
    "rest_framework",
    "rest_framework_simplejwt",
    #
    "web",
    "airspacecheck",
    "running",
    "kudosdeluxe",
    #
    # all-auth
    "allauth",
    "allauth.account",
    "allauth.socialaccount",
    "allauth.socialaccount.providers.apple",
    "allauth.socialaccount.providers.strava",
]

REST_FRAMEWORK = {
    "DEFAULT_AUTHENTICATION_CLASSES": (
        "rest_framework_simplejwt.authentication.JWTAuthentication",
    ),
}

SIMPLE_JWT = {
    "ACCESS_TOKEN_LIFETIME": timedelta(minutes=5),  # Adjust as needed
    "REFRESH_TOKEN_LIFETIME": timedelta(days=1),
    # ... other settings ...
}

MIDDLEWARE = [
    "web.middleware.HealthCheckMiddleware",  # health check
    #
    "django.middleware.security.SecurityMiddleware",
    "django.contrib.sessions.middleware.SessionMiddleware",
    "django.middleware.common.CommonMiddleware",
    "django.middleware.csrf.CsrfViewMiddleware",
    "django.contrib.auth.middleware.AuthenticationMiddleware",
    "django.contrib.messages.middleware.MessageMiddleware",
    "django.middleware.clickjacking.XFrameOptionsMiddleware",
    # Add the account middleware:
    "allauth.account.middleware.AccountMiddleware",
]

ROOT_URLCONF = "geder.urls"

TEMPLATES = [
    {
        "BACKEND": "django.template.backends.django.DjangoTemplates",
        "DIRS": [TEMPLATE_DIR],
        "APP_DIRS": True,
        "OPTIONS": {
            "context_processors": [
                "django.template.context_processors.debug",
                "django.template.context_processors.request",
                "django.contrib.auth.context_processors.auth",
                "django.contrib.messages.context_processors.messages",
            ],
        },
    },
]

WSGI_APPLICATION = "geder.wsgi.application"

AUTHENTICATION_BACKENDS = [
    # Needed to login by username in Django admin, regardless of `allauth`
    "django.contrib.auth.backends.ModelBackend",
    # `allauth` specific authentication methods, such as login by email
    "allauth.account.auth_backends.AuthenticationBackend",
]

# Allauth Configuration
# LOGIN_REDIRECT_URL = "/"
ACCOUNT_AUTHENTICATION_METHOD = "username_email"  # Or 'email' if preferred
ACCOUNT_EMAIL_REQUIRED = True
ACCOUNT_EMAIL_VERIFICATION = "mandatory"
LOGIN_REDIRECT_URL = "kudosdeluxe:link"
LOGOUT_REDIRECT_URL = "web:login"

# Add the client ID and secret for Strava (to be set later)
STRAVA_CLIENT_ID = MY_STRAVA_CLIENT_ID
STRAVA_CLIENT_SECRET = MY_STRAVA_CLIENT_SECRET
STRAVA_REDIRECT_URI = BASE_URL + "/kudosdeluxe/callback/"

# Database
# https://docs.djangoproject.com/en/5.0/ref/settings/#databases

DATABASES = {
    "default": {
        "ENGINE": "django.db.backends.sqlite3",
        "NAME": BASE_DIR / "db.sqlite3",
    }
}

# SOCIALACCOUNT_PROVIDERS = {
#     "apple": {
#         "APP": {
#             "client_id": APPLE_CLIENT_ID,
#             "team_id": APPLE_TEAM_ID,
#             "key_id": APPLE_KEY_ID,
#             "secret": "",  # Contents of the .p8 file
#         },
#         "SCOPE": [
#             "email",
#             "name",
#         ],
#         "AUTH_PARAMS": {
#             "response_mode": "form_post",
#         },
#         "USE_JWT": True,
#     }
# }


SOCIALACCOUNT_PROVIDERS = {
    "apple": {
        "APPS": [
            {
                # Your Apple Services ID
                "client_id": APPLE_CLIENT_ID,
                # Your Apple Key ID
                "secret": APPLE_KEY_ID,
                # Your Apple Team ID (App ID Prefix)
                "key": APPLE_TEAM_ID,
                # The private key content
                "settings": {
                    "certificate_key": open(
                        os.path.join(BASE_DIR, APPLE_PRIVATE_KEY_PATH)
                    ).read(),
                },
            }
        ]
    },
    "strava": {
        "APP": {
            "client_id": STRAVA_CLIENT_ID,
            "secret": STRAVA_CLIENT_SECRET,
            "key": "",
        }
    },
}


AUTH_PASSWORD_VALIDATORS = [
    {
        "NAME": "django.contrib.auth.password_validation.UserAttributeSimilarityValidator",
    },
    {
        "NAME": "django.contrib.auth.password_validation.MinimumLengthValidator",
    },
    {
        "NAME": "django.contrib.auth.password_validation.CommonPasswordValidator",
    },
    {
        "NAME": "django.contrib.auth.password_validation.NumericPasswordValidator",
    },
]


# Internationalization
# https://docs.djangoproject.com/en/5.0/topics/i18n/

LANGUAGE_CODE = "en-us"

TIME_ZONE = "UTC"

USE_I18N = True

USE_TZ = True


# Static files (CSS, JavaScript, Images)
# https://docs.djangoproject.com/en/5.0/howto/static-files/

STATIC_URL = "static/"

# Default primary key field type
# https://docs.djangoproject.com/en/5.0/ref/settings/#default-auto-field

DEFAULT_AUTO_FIELD = "django.db.models.BigAutoField"

LOGGING = {
    "version": 1,
    "disable_existing_loggers": False,
    "formatters": {
        "detailed": {
            "format": "%(asctime)s - %(module)s - %(funcName)s - %(levelname)s - %(message)s"
        },
    },
    "handlers": {
        "file": {
            "level": LOGGING_LEVEL,
            "class": "logging.FileHandler",
            "filename": BASE_DIR / LOG_FILE_PATH,
            "formatter": "detailed",
        },
    },
    "loggers": {
        "": {
            "handlers": ["file"],
            "level": LOGGING_LEVEL,
            "propagate": True,
        },
    },
}
